@@ -42,6 +42,8 @@ class AutoBlueprint:
         names and routes dict
     index_page : str
         html template for blueprint index
+    index_menu : str
+        label for index_page in menu
     _import_route : bool
         enable db import
     _export_route : bool
@@ -55,9 +57,9 @@ class AutoBlueprint:
     """
     
     def __init__(
-            self, app=None, registry=registry(), url_prefix=None, enable_crud=False,
+            self, app=None, registry=None, url_prefix=None, enable_crud=False,
             fair_data=True, forensics=False, protect_data=True, imex=True,
-            protect_index=False, index_page='base.html'):
+            protect_index=False, index_page='base.html', index_menu=None):
         """
         Parameters
         ----------
@@ -75,10 +77,13 @@ class AutoBlueprint:
             names and routes dict
         index_page : str
             html template for blueprint index        
+        index_menu : str
+            label for index_page in menu
         """
         
         self.name = self.__class__.__name__.lower()
         self.fair_data = fair_data
+        self.enable_crud = enable_crud
         self.forensics = forensics # track user and time of modifications
         self.protect = protect_data
         self.protect_index = protect_index
@@ -87,13 +92,17 @@ class AutoBlueprint:
         self.routes = OrderedDict()
         self.view_functions = {}
         self.index_page = index_page
+        self.index_menu_name = index_menu or self.name
         if imex:
             self._import_route = True
             self._export_route = True
 
         # Set up logging
         self.logger = logging.getLogger(self.name)
+        if registry:
+            self.set_bauprint(registry)
 
+    def set_bauprint(self, registry):
         # Set up registry
         self.mapper_registry = registry# or self.registry
         # To allow sibling AutoBlueprint's to find models in each other's namespaces
@@ -112,13 +121,12 @@ class AutoBlueprint:
 
         # Routes
         self.add_routes()
-        if enable_crud:
-            self.enable_crud = self.models if enable_crud is True else enable_crud
+        if self.enable_crud:
+            self.enable_crud = self.models if self.enable_crud is True else self.enable_crud
             self.add_crud_routes()
         self.add_url_rules()
 
-
-    def init_app(self, app):
+    def init_app(self, app, registry=None):
         """
         Parameters
         ----------
@@ -250,34 +258,9 @@ class AutoBlueprint:
             class ModelForm(FlaskForm):
                 pass
 
-            for fieldname, fieldtype in dm.__annotations__.items():
-                if fieldtype == list[int]: continue
-                default_value = dm.__bauprint__[fieldname].default
-                if fieldname.endswith('_id') and fieldname[:-3] in self.all_models:
-                    model = self.all_models[fieldname[:-3]]
-                    setattr(
-                        ModelForm,
-                        fieldname,
-                        SelectField(
-                            fieldname.replace('_',' ').capitalize(),
-                            # lambda required default model as otherwise the last reference to model is used
-                            choices=lambda model=model: [(i.id,i) for i in self.db.session.query(model).all()]
-                        )
-                    ) # TODO allow blank option if default is None
-                else: setattr(
-                    ModelForm,
-                    fieldname,
-                    # Primitive types
-                    BauType._get_bautype(fieldtype).ux_type(
-                        fieldname.replace('_',' ').capitalize(),
-                        validators=(
-                            [] if fieldtype is bool else [
-                                Optional() if default_value is None else InputRequired()
-                            ]
-                        ),
-                        default=None if default_value is MISSING else default_value
-                    )
-                )
+            for bauhaus in dm.__bauprint__.values():
+                bauhaus.build_ux_field(self, ModelForm)
+                
             setattr(ModelForm, 'submit', SubmitField(f'Submit "{name}"'))
             self.forms[name.lower()] = ModelForm
 
@@ -288,7 +271,7 @@ class AutoBlueprint:
         for name, dm in self.datamodels.items():
             self.model_properties[name] = {}
             columns = {
-                colname: coltype.create_db_column(self)
+                colname: coltype.build_db_column(self)
                 for colname, coltype in dm.__bauprint__.items()
             }
             if self.forensics:
@@ -365,7 +348,8 @@ class AutoBlueprint:
                 f"{name}_create", f"/{name}/create",
                 roles=[self.name],
                 view_function=self.create, defaults={'name':name},
-                methods=('GET','POST'), menu_label=f"Create {name}"
+                methods=('GET','POST'),
+                menu_label=f"Create {name}"
             )
             # List
             self.add_route(
@@ -373,7 +357,7 @@ class AutoBlueprint:
                 roles=not(self.fair_data),
                 view_function=self.list,
                 defaults={'name':name},
-                menu_label=f"List {name}"
+                menu_label=name.capitalize().replace('_',' ')
             )
             # Read
             self.add_route(
@@ -456,28 +440,39 @@ class AutoBlueprint:
             self.routes[self.name] = Route(
                 '/', self.name,
                 roles=False if self.fair_data else [self.name],
-                view_function=self.index, menu_label=self.name
+                view_function=self.index, menu_label=self.index_menu_name
             )
         cls = self.__class__
         viewfunctions = inspect.getmembers(
             cls,
             lambda x: inspect.isroutine(x)
             and hasattr(x,'__annotations__')
-            and isinstance(
+            and (isinstance(
                 x.__annotations__.get('return',None),
-                (str,Route) #TODO include dict for json returning routes
-            )
+                (str,Route) #Instance in case route url is passed directly or Route instance
+            ) or x.__annotations__.get('return',None) in (str,dict,Route))
         )
         for name, viewfunction in viewfunctions:
-            self.routes[f"/{name}"] = Route(
-                f"/{name}", name,
-                roles=False,
-                view_function=viewfunction,
-                defaults={'self':self}
-            ) if isinstance(
-                viewfunction.__annotations__.get('return'), str
-            # If explicit Route is defined use that
-            ) else viewfunction.__annotations__.get('return')
+            return_type = viewfunction.__annotations__.get('return')
+            if isinstance(return_type, str):
+                self.routes[name] = Route(
+                    return_type, name,
+                    roles=False,
+                    view_function=getattr(self,name),
+                    return_type=str
+                )
+            elif inspect.isclass(return_type) and issubclass(return_type, (str,dict)): # representing abstract html or json
+                self.routes[name] = Route(
+                    f"/{name}", name,
+                    roles=False,
+                    view_function=getattr(self,name),
+                    return_type=return_type
+                )
+            elif isinstance(return_type, Route):
+                # If explicit Route is defined use that but set name and function
+                return_type.set_view_post_init(name,getattr(self,name)) #using viewfunction requires providing self arg
+                #return_type.defaults={'self':self}
+                self.routes[name] = return_type
     
     def add_url_rules(self):
         for route in self.routes.values():
@@ -504,7 +499,7 @@ class AutoBlueprint:
     def index(self):
         return render_template(self.index_page)
         
-    def create(self, name):
+    def create(self, name, success_redirect=None, success_template=None, success_template_kwargs=None):
         form = self.forms[name]()
         setattr(form, 'submit', SubmitField(f'Submit "{name}"'))
         if form.validate_on_submit():
@@ -521,7 +516,12 @@ class AutoBlueprint:
 
             flash(f"{name} instance was created")
 
-            return redirect(url_for(f"{self.name}_blueprint.{name}_list"))
+            if success_redirect:
+                return redirect(success_redirect)
+            elif success_template:
+                return render_template(success_template, **success_template_kwargs)
+            else:
+                return redirect(url_for(f"{self.name}_blueprint.{name}_list"))
         return render_template('uxfab/form.html', form=form, title=f"Create {name}")
 
     def list(self, name):
@@ -543,7 +543,9 @@ class AutoBlueprint:
                 title=f"{list_attribute.capitalize()} of {name}"
             )
 
-    def update(self, name, id, list_attribute=None):
+    def update(
+            self, name, id, list_attribute=None, success_redirect=None,
+            success_template=None, success_template_kwargs=None):
         item = self.db.session.query(self.models[name]).get_or_404(id)
         if list_attribute:
             list_model_name = list_attribute[:-len('_list')]
@@ -564,8 +566,13 @@ class AutoBlueprint:
                 self.db.session.commit()
                 
                 flash(f"{name} instance was created")
-                
-                return redirect(url_for(f"{self.name}_blueprint.{name}_list"))
+
+                if success_redirect:
+                    return redirect(success_redirect)
+                elif success_template:
+                    return render_template(success_template, **success_template_kwargs)
+                else:
+                    return redirect(url_for(f"{self.name}_blueprint.{name}_list"))
             name = list_attribute
 
         # Normal update
@@ -595,7 +602,12 @@ class AutoBlueprint:
                 
                 flash(f"{name} instance was updated")
                 
-                return redirect(url_for(f"{self.name}_blueprint.{name}_read", id=item.id))
+                if success_redirect:
+                    return redirect(success_redirect)
+                elif success_template:
+                    return render_template(success_template, **success_template_kwargs)
+                else:
+                    return redirect(url_for(f"{self.name}_blueprint.{name}_read", id=item.id))
         return render_template('uxfab/form.html', form=form, title=f"Update {name} for {item}")
 
     def delete (self, name, id):
